import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

import java.util.*;

/**
 * An explosion. It starts by expanding and then collapsing. 
 * The explosion will explode other obejcts that the explosion intersects.
 * 
 * @author Poul Henriksen
 * @version 1.0.1
 */
public class Explosion extends Actor
{
    /** How many images should be used in the animation of the explostion */
    private final static int IMAGE_COUNT= 8;
    
    /** 
     * The images in the explosion. This is static so the images are not
     * recreated for every object (improves performance significantly).
     */
    private static GreenfootImage[] images;
    
    /** Current size of the explosion */
    private int imageNo = 0;
    
    /** How much do we increment the index in the explosion animation. */
    private int increment=1;
    private boolean foundTarget = false;
    private static final int DAMAGE_TO_BOSS=5;
    private static final int AREA=8;
    private static final int AREA_FOR_BOSS=15;
    private static final int SPEED=8;
    
    public Explosion() {
        initialiseImages();
        setImage(images[0]);        
        Greenfoot.playSound("Explosion.wav");
    }    
    
    /** 
     * Create the images for explosion.
     */
    public synchronized static void initialiseImages() {
        if(images == null) {
            GreenfootImage baseImage = new GreenfootImage("explosion-big.png");
            int maxSize = baseImage.getWidth();
            int delta = maxSize / IMAGE_COUNT;
            int size = 0;
            images = new GreenfootImage[IMAGE_COUNT];
            for(int i=0; i < IMAGE_COUNT; i++) {
                size = size + delta;
                images[i] = new GreenfootImage(baseImage);
                images[i].scale(size, size);
            }
        }
    }
    
    /**
     * EXPLODE!
     */
    public void act()
    { 
        setImage(images[imageNo]);
        theCopIntersection();
        theMatIntersection();
        imageNo += increment;
        if(imageNo >= IMAGE_COUNT) {
            increment = -increment;
            imageNo += increment;
        }
        
        if(imageNo < 0) {
            getWorld().removeObject(this);
        }
    }     
    /**
     * Check if intersects with a cop.
     */ 
    public void theCopIntersection(){
        if(!foundTarget){
            List<Cop> copList = getIntersectingObjects(Cop.class);
            if(!copList.isEmpty() && copList!=null){
                
                for(int i =0;i<copList.size();i++){
                    Cop cop = (Cop)copList.get(i);
                    if(!cop.isDead){
                        foundTarget = true;
                        cop.isDead = true;
                        cop.die();
                        break;
                    }
                }
            }
        }
    }
    /**
     * Check if intersects with a mat.
     */
    public void theMatIntersection(){
        if(!foundTarget){
            List<Mat> matList = getIntersectingObjects(Mat.class);
            if(!matList.isEmpty() && matList!=null){
                
                for(int i =0;i<matList.size();i++){
                    Mat mat = (Mat)matList.get(i);
                    
                    if(!mat.isDead){
                        foundTarget = true;
                        mat.isDead = true;
                        mat.die();
                        break;
                    }
                }
            }
        }
    }
}