import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
import java.util.List;
import java.util.ArrayList;
/**
 * Write a description of class Jack here.
 * 
 * @author Tzanis Aristeidis
 * @version 1.0
 */
public class Jack  extends Mover
{
    private Vector upVector;//the vectors for the players movement.
    private Vector downVector;
    private boolean shooting;//checks if the firing animation is finished(false).
    private boolean isBossOut;
    private GreenfootImage[] firingChair;//the set of images for the firing animation.
    private GreenfootImage[] firingDesk;
    private GreenfootImage[] firingBomb;
    private GreenfootImage[] firingGun;
    private List<String> inventory;
    private String currentWeapon;
    private int[] times;
    private JackHealth health;

    private int counter;//the counter that delays the animation.
    private int imageNo;//image number for the firing animation.
    private int currentWeap =0;
    
    private int gunReloadTime;//how fast is the player shooting.  
    private int theScore;//an integer for the players score.
    private int playersLevel;//an integer for the players level.
    private int theMoney;
    
    private GreenfootImage moneyImage;//the image that shows the players money on the panel.
    private GreenfootImage scoreImage;//the image that shows the players score on the panel.
    private GreenfootImage levelImage;//the image that shows the players level on the panel.
    
    /**
     * The constructor.
     */
    public Jack()
    {
        this.times = new int[3];
        this.currentWeapon = "chair";
        this.inventory = new ArrayList();
        this.inventory.add("chair");
        this.downVector = new Vector(90, 3);
        this.upVector = new Vector(-90, 3);
        this.shooting= false;
        this.theScore =  0;
        this.theMoney= 0;
        this.playersLevel=1;
        this.gunReloadTime = 2;
        this.isBossOut = false;
        setupChairAnimationToShoot(6);
        setupDeskAnimationToShoot(6);
        setupBombAnimationToShoot(6);
        setupGunAnimationToShoot(6);
    }

    /**
     * The act method.
     */
    public void act() 
    {
        TheStreet ts = (TheStreet)getWorld();
        if(!ts.isShopOpen){
            checkKeys();
            if (shooting == true)fireAnimation(gunReloadTime, getCurrentWeapon());
        }
    }
    
    /**
     * Sets the reload time.
     * @theTime as less the time is as fast the player is shooting.
     */
    private void setReloadTime(int theTime){
        this.gunReloadTime = theTime;
    }
    
    /**
     * Sets the players score
     * @score The score.
     */
    public void setScore(int score){
        scoreImage = getWorld().getBackground();
        Font font = scoreImage.getFont();
        font = font.deriveFont(12.0f);
        scoreImage.setFont(font);
        scoreImage.setColor(new Color(76, 74, 44));
        scoreImage.fillRect(890,22,55,15);
        scoreImage.setColor(Color.WHITE);
        scoreImage.drawString(String.valueOf(score), 910, 34);
        this.theScore = score;
        
        //Set the score to increase the levels.
        if (score>=10&&score<=29) increaseLevel(2);
        if (score>=30&&score<=49) increaseLevel(3);
        if (score>=50&&score<=69) increaseLevel(4);
        if (score>=70&&score<=89) increaseLevel(5);
        if (score>=90&&score<=119)increaseLevel(6);
        if (score>=120&&score<=139) increaseLevel(7);
        if (score>=140&&score<=159){
            if(!isBossOut)showHealthBar();
            increaseLevel(8);
            isBossOut = true;}
        if (score>=160&&score<=180) increaseLevel(9);
    }
    
    public void setMoney(int money){
        int x = 845;
        //To make the money look better.
        if(money>=100)x=838;
        if(money>=1000)x=831;
        
        moneyImage = getWorld().getBackground();
        Font font = moneyImage.getFont();
        font = font.deriveFont(13.0f);
        moneyImage.setFont(font);
        moneyImage.setColor(new Color(76, 74, 44));
        moneyImage.fillRect(x, 140,35,15);
        moneyImage.setColor(Color.YELLOW);
        moneyImage.drawString(String.valueOf(money),x+15, 154);
        this.theMoney = money;      
    }
    
    public void setCurrentWeapon(String weapon){
        this.currentWeapon= weapon;
    }
    
    public void addBullets(String weapon, int numOfBullets){
        if (weapon =="desk"){
            times[0]+=numOfBullets;
        }
        if (weapon =="bomb"){
            times[1]+=numOfBullets;
            
        }
        if (weapon =="gun"){
            times[2]+=numOfBullets;
            
        }
    }
    
    public void addItemToInventory(String itemName){
        if(!inventory.contains(itemName)) inventory.add(itemName);
    }
    
    public int getHealth(){
        return health.getHealth();
    }
    
    public int getBullets(String weapon){
        if (weapon =="desk"){
            return times[0];
        }
        if (weapon =="bomb"){
            return times[1];
        }
        if (weapon =="gun"){
            return times[2];
        }return 0;
    }
    
    public int getLevel(){
        return playersLevel;
    }
    
    public String getCurrentWeapon(){
        return this.currentWeapon;
    }
    
    public List<String> getInventory(){
        return this.inventory;
    }
    /**
     * Returns the players money.
     */    
    public int getMoney(){
        return this.theMoney;
    }
    
    /**
     * Returns the players score.
     */
    public int getScore(){
        return this.theScore;
    }
    
    /**
     * Puts all the images needed for the firing animation.
     * @frames The number of images needed for the animation.
     */
    public void setupChairAnimationToShoot(int frames){
        
        firingChair = new GreenfootImage[frames];
        firingChair[0] = new GreenfootImage("images/theJackThrowing1chair.png");
        firingChair[1] = new GreenfootImage("images/theJackThrowing2chair.png");
        firingChair[2] = new GreenfootImage("images/theJackThrowing3chair.png");
        firingChair[3] = new GreenfootImage("images/theJackThrowing3.png");
        firingChair[4] = new GreenfootImage("images/theJackThrowing2.png");
        firingChair[5] = new GreenfootImage("images/theJackThrowing1chair.png");
    }
    public void setupDeskAnimationToShoot(int frames){
        firingDesk = new GreenfootImage[frames];
        firingDesk[0] = new GreenfootImage("images/theJackThrowing1desk.png");
        firingDesk[1] = new GreenfootImage("images/theJackThrowing2desk.png");
        firingDesk[2] = new GreenfootImage("images/theJackThrowing3desk.png");
        firingDesk[3] = new GreenfootImage("images/theJackThrowing3.png");
        firingDesk[4] = new GreenfootImage("images/theJackThrowing2.png");
        firingDesk[5] = new GreenfootImage("images/theJackThrowing1desk.png");
    }
    public void setupBombAnimationToShoot(int frames){
        firingBomb = new GreenfootImage[frames];
        firingBomb[0] = new GreenfootImage("images/theJackThrowing1bomb.png");
        firingBomb[1] = new GreenfootImage("images/theJackThrowing2bomb.png");
        firingBomb[2] = new GreenfootImage("images/theJackThrowing3bomb.png");
        firingBomb[3] = new GreenfootImage("images/theJackThrowing3.png");
        firingBomb[4] = new GreenfootImage("images/theJackThrowing2.png");
        firingBomb[5] = new GreenfootImage("images/theJackThrowing1bomb.png");
    }
    public void setupGunAnimationToShoot(int frames){
        firingGun = new GreenfootImage[frames];
        firingGun[0] = new GreenfootImage("images/Jack/jackShooting.png");
        firingGun[1] = new GreenfootImage("images/Jack/jackShooting.png");
        firingGun[2] = new GreenfootImage("images/Jack/jackShooting.png");
        firingGun[3] = new GreenfootImage("images/Jack/jackShooting.png");
        firingGun[4] = new GreenfootImage("images/Jack/jackShooting.png");
        firingGun[5] = new GreenfootImage("images/Jack/jackShooting.png");
    }
    
    /**
     * Checks the keys pressed.
     */
    public void checkKeys()
    {
        if (Greenfoot.isKeyDown("up")){
            if (getY()>270){
                setSpeed(upVector);
                if(health!=null)health.setLocation(840, getY()-45);
                move();
            }
        }
        if (Greenfoot.isKeyDown("down")){
            if (getY()<352){
                setSpeed(downVector);
                if(health!=null)health.setLocation(840, getY()-35);
                move();
            }
        }
        if (Greenfoot.isKeyDown("right")){
            nextItem();
        }
        if (Greenfoot.isKeyDown("space")){
            shooting = true;
        }
    }
    
    /**
     * Executes the fire animation
     * @speed How fast the animation is going to be.
     * @weapon The weapon that is used.
     */
    public void fireAnimation(int speed, String weapon){
        GreenfootImage[] firing = new GreenfootImage[6];        
        if (weapon == "chair"){
            for(int i = 0; i<6; i++){
                this.gunReloadTime = 3;
                firing[i] = firingChair[i];}
        }
        if (weapon == "desk"){
            for(int i = 0; i<6; i++){
                this.gunReloadTime = 2;
                firing[i] = firingDesk[i];}
        }        
        if (weapon == "bomb"){
            for(int i = 0; i<6; i++){
                this.gunReloadTime = 2;
                firing[i] = firingBomb[i];}
        }        
        if (weapon == "gun"){
            for(int i = 0; i<6; i++){
                this.gunReloadTime = 1;
                firing[i] = firingGun[i];}
        }
        //TODO: reduce frames for shooting.
        if (counter%speed==0){
            setImage(firing[imageNo]);
            imageNo ++;
            if (imageNo==4)
                fire(getCurrentWeapon());
            if (imageNo==firing.length){
                shooting = false;
                imageNo = 0;
            }
        }counter++;
    }

    /**
     * Called when the animation is finished, to create the object to throw.
     * @weapon The weapon used.
     */
    public void fire(String weapon){
        if(weapon == "chair"){
            Chair chair = new Chair(getMovement().copy());
            getWorld().addObject(chair, getX()-60, getY());
            chair.move();
        }else if(weapon == "desk"){
            Desk desk = new Desk(getMovement().copy());
            getWorld().addObject(desk, getX()-60, getY());
            desk.move();
            times[0]-=1;
            printTimes(times[0]);
        }else if(weapon == "bomb"){
            Bomb bomb = new Bomb(getMovement().copy());
            getWorld().addObject(bomb, getX()-60, getY()-22);
            bomb.move(); 
            times[1]-=1;
            printTimes(times[1]);
        }else {
            Bullet bullet = new Bullet(getMovement().copy());
            getWorld().addObject(bullet, getX()-60, getY()-22);
            bullet.move();
            times[2]-=1;
            printTimes(times[2]);
        }
    }
    
    /**
     * Selects the next item.
     */
    public void nextItem(){
        int invSize = getInventory().size();
        try{
            if (invSize>1){
                if(currentWeap<invSize){
                    if (currentWeap == invSize-1){
                        currentWeap = 0;
                        currentWeapon(getInventory().get(currentWeap));
                        setCurrentWeapon(getInventory().get(currentWeap));
                    }else{ 
                        currentWeap++;
                        currentWeapon(getInventory().get(currentWeap));
                        setCurrentWeapon(getInventory().get(currentWeap));
                        printTimes(getBullets(getInventory().get(currentWeap)));
                    }
                }else{
                    currentWeap = 0;
                    currentWeapon(getInventory().get(currentWeap));
                    setCurrentWeapon(getInventory().get(currentWeap));}
            }else{
                    currentWeap = 0;
                    currentWeapon(getInventory().get(currentWeap));
                    setCurrentWeapon(getInventory().get(currentWeap));
            }
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    
    /**
     * Prints out the current weapon.
     * @curWeap The weapon to display.
     */
    public void currentWeapon(String curWeap){
        try{
            GreenfootImage weaponImage = null;
            int x=893;
            if(curWeap == "chair"){
                weaponImage = new GreenfootImage("images/chairThumb.png");
                printTimes(10000000);
            }            
            if(curWeap == "desk")
                weaponImage = new GreenfootImage("images/deskThumb.png");
            if(curWeap == "bomb")
                weaponImage = new GreenfootImage("images/bombThumb.png");
            if(curWeap == "gun"){
                weaponImage = new GreenfootImage("images/gunThumb.png");
                x=875;
            }
            getWorld().getBackground().setColor(new Color(76, 74, 44));
            getWorld().getBackground().fillRect(875,60,61,43);
            getWorld().getBackground().drawImage(weaponImage,x,60);
            
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    
    /**
     * Prints the times the weapon is available.
     * @times The bullets or the chairs etc.
     */
    public void printTimes(int times){
        GreenfootImage timesImage = getWorld().getBackground();
        GreenfootImage inf = new GreenfootImage("images/infinit.png");
        Font font = timesImage.getFont();
        font = font.deriveFont(12.0f);
        timesImage.setFont(font);
        timesImage.setColor(new Color(76, 74, 44));
        timesImage.fillRect(890,100,65,22);
        timesImage.setColor(Color.WHITE);
        if(times<=0){
            this.inventory.remove(this.currentWeapon);
            nextItem();
        }
        else if(times<10000000&&times>0)
            timesImage.drawString(String.valueOf(times), 910, 118);    
        else
            timesImage.drawImage(inf, 910, 110); 
    }
    
    /**
     * Each time is called sets the players level.
     * @level Thelevel to set.
     */
    private void increaseLevel(int level){
        levelImage = getWorld().getBackground();
        Font font = scoreImage.getFont();
        font = font.deriveFont(12.0f);
        levelImage.setFont(font);
        playersLevel=level;
        levelImage.setColor(new Color(76, 74, 44));
        levelImage.fillRect(890,40,35,15);
        levelImage.setColor(Color.WHITE);
        levelImage.drawString(String.valueOf(playersLevel), 910, 53);
    }
    
    /**
     * Creates a new JackHealth.
     */
    public void showHealthBar(){
        health = new JackHealth();
        getWorld().addObject(health,840,getY()-35);
    }
    
    /**
     * Decreases the health calling the hit in JackHealth.
     * @damage The damage to cause.
     */
    public void hit(int damage){
        health.hit(damage);
    }
}
