import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.awt.Font;

/**
 * The ScoreBoard is used to display results on the screen. It can display some
 * text and several numbers.
 * 
 * @author Tzanis Aristeidis
 * @version 1.0
 */
public class ScoreBoard extends Actor
{
    public static final float FONT_SIZE = 24.0f;
    public static final int WIDTH = 300;
    public static final int HEIGHT = 200;
    
    public ScoreBoard()
    {
        makeImage("Scores", "Score: ", "nobody", 100);
    }
    /**
     * Create a score board for the final result.
     * @title The title for the scoreboard.
     * @text A small text for the scoreboard.
     * @prefix A prefix for the score.
     * @score The score.
     */
    public ScoreBoard(String title, String text, String prefix, int score)
    {
        int total = score;
        makeImage(title, text, prefix, total);
    }
    
    /**
     * Make the score board image.
     * @title The title for the scoreboard.
     * @text A small text for the scoreboard.
     * @prefix A prefix for the score.
     * @score The score.
     */
    private void makeImage(String title, String text, String prefix, int score)
    {
        GreenfootImage image = new GreenfootImage(WIDTH, HEIGHT);

        image.setColor(new Color(0, 0, 0, 128));
        image.fillRect(0, 0, WIDTH, HEIGHT);
        image.setColor(new Color(0, 0, 0, 128));
        image.fillRect(5, 5, WIDTH-10, HEIGHT-10);
        Font font = image.getFont();
        font = font.deriveFont(FONT_SIZE);
        image.setFont(font);
        image.setColor(Color.WHITE);
        image.drawString(title, 60, 50);
        image.drawString(text, 60, 90);
        image.drawString(prefix + score, 60, 170);
        setImage(image);
    }
}
